%%% takes in abf file ('expt') of 10ms current injection steps of increasing
%%% amplitude, plots phase plane plots and determines spike waveform
%%% measures.  Default takes measures from first spike fired at current
%%% threshold; alternatively can choose to take measures from sweep defined
%%% in 'sweep' input.  By defining 'txt' input, can choose to output AP
%%% trace to txt file for analysis in Maxim Volgushev's 'FitAPs' script


function [] = galliano_AP_10ms(expt,sweep,txt)

close all

th = 0;  %%% threshold for defining AP, usually 0

pAstart = 0;
pAstep = 10;

[d,si] = abf2load(expt);  %%% loads in data from .abf file into matrix d, with sample interval si (in microsec)
%%%% structure of d
%%%% 1st dimension is data points in sweep, 2nd is no of channels, 3rd is
%%%% no of sweeps
s = size(d) ;
nsweeps = s(3);

disp(si)


for j = 1:nsweeps
    
    if s(2) == 1 % so if output is only in voltage channel; requires pAstep & pAstart inputs, and can only deal in relative current changes
        
        t{j} = [0:si:(si*(s(1)-1))]'./1000; %%% time#1 array; doesn't need to be built on sweep-by-sweep basis, but is safest.  Does need to be inverted to match voltage array from d
        v{j} = d(:,1,j); %%% voltage array      
        t2{j} = t{j}; %%% time#2 array; stupid to have this here, but safest for historical Heka analysis reasons
        
        istep(j) = pAstart + ((j-1)*pAstep);
        
        %plot(t{n},v{n})
        
        %pause
        
    elseif s(2) == 2
        
        t{j} = [0:si:(si*(s(1)-1))]'./1000; %%% time#1 array; doesn't need to be built on sweep-by-sweep basis, but is safest.  Does need to be inverted to match voltage array from d
        v{j} = (d(:,1,j)); %%% voltage array      
        t2{j} = t{j}; %%% time#2 array; stupid to have this here, but safest for historical Heka analysis reasons
        a{j} = (d(:,2,j)./1000000000000);
        
       
        bas_i = find(t{j}<10);
        
        step_i = find((t{j}>13)&(t{j}<18));
        hold_a(j) = mean(a{j}(bas_i))*1000000000000;
        istep(j) = round(mean(a{j}(step_i))*1000000000000 - hold_a(j));
        
        %istep(j) = pAstart + ((j-1)*pAstep);
        %%% build up arrays in here for dual-channel recordings with both v
        %%% and amps
        
        
    
    end
    
    plot(t{j},v{j})
    hold on
    
    tstep(j) = mean(diff(t{j}));
    
    vhold(j) = mean(v{j}(1:1950));
    plot([t{j}(1) t{j}(1950)],[vhold(j) vhold(j)],'m-')
    disp(['Trace ' num2str(j) ' I ' num2str(istep(j)) ' Vhold ' num2str(vhold(j))])   
        
    
    [vmax(j) vmax_i(j)] = max(v{j});
    
    if vmax(j)>th
        disp('spike!')
        spikes(j) = 1;
    else
        spikes(j) = 0;
    end
    
    %pause
    
end

tstep = mean(tstep);
%disp(tstep.*1000)

%%%%%%%%% spike waveform measures from 1st AP

if nargin>1
    j = sweep;
else
    j = find(spikes>0); j = j(1);
end
ts = t{j};
vs = v{j};

figure(2)
subplot(5,1,1)
plot(ts,vs)
hold on

[vmax vmaxi] = max(vs);
vmaxt = ts(vmaxi);
plot(vmaxt,vmax,'g*')

    %%%%%%%%%% finding threshold & other measures
    
window = 20; %%% size of sliding window
a = 1; b = ones(1,window)./window;
vf = filter(b,a,vs); %%% sliding mean
vf = vf(100:length(vf));

    %%%%% saving voltage trace to txt file for FitAPs analysis (Volgushev fits)


    if nargin>2
        savefile = [expt(1:(length(expt)-4)) '_' num2str(j) '.txt'];
        fid = fopen(savefile,'at');
        for n = 1:length(vf)
            fprintf(fid,'%f\n',vf(n));
        end
        fclose(fid);
    end


    %%%%%%%

ts = ts(100:length(ts));
tf = ts-(tstep.*(window-1)/2);
vs = vs(100:length(vs));
plot(tf,vf,'m-')
    
dv = diff(vf);
dvdt = dv ./ tstep;   %%% 1st derivative of smoothed trace
t1 = tf(1:length(tf)-1);    
subplot(5,1,2)
plot(t1,dvdt)
xlabel('s');ylabel('V/s')
hold on    
vf2 = filter(b,a,dvdt);
plot(t1-(tstep.*(window-1)/2),vf2,'m-')
[mx_dvdt mxi_dvdt] = max(dvdt);
tmx_dvdt = t1(mxi_dvdt);    
title(['max dV/dt ' num2str(mx_dvdt)])
plot(tmx_dvdt,mx_dvdt,'*c')
tsh_i = find((t1<tmx_dvdt)&(dvdt<10));    %%% so threshold where 1st derivative goes above 10mV/ms
tshi = tsh_i(length(tsh_i));
plot(t1(tshi),10,'g*')
t_thresh = ts(tshi);    
v_thresh = vs(tshi);
subplot(5,1,1)
plot(t_thresh,v_thresh,'g*')
hh = v_thresh+ (0.5.*(vmax-v_thresh));
hhv1i = find((ts>t_thresh)&(vs>=hh)); hh1i = hhv1i(1);
hhv2i = find((ts>vmaxt)&(vs>=hh));
hh2i = hhv2i(length(hhv2i));
hhv1t = ts(hh1i);
hhv2t = ts(hh2i);
plot([hhv1t hhv2t],[hh hh],'g:')
whh = hhv2t-hhv1t;
title(['vthresh ' num2str(v_thresh) '  vmax ' num2str(vmax) '  whh ' num2str(whh)])
hold off

%%%% phase plot, with alternative Vth, and onset rapidness measures

cuti = find(t1>15);  %%% so dispensing with initial transient
v_pp = vf(cuti);
dvdt_pp = dvdt(cuti);

figure(3)
subplot(2,2,1)
plot(v_pp,dvdt_pp)
axis square
hold on


tsh_i = find(dvdt_pp>=10);
okcount = 0;
for k = 1:(length(tsh_i)-10)
    if min(dvdt_pp(tsh_i(k):tsh_i(k)+10))>=10
        okcount = okcount+1;
        tishi(okcount) = tsh_i(k);
    end
end
tshi = tishi(1);
vth_pp = v_pp(tshi);
dvdt_vth_pp = dvdt_pp(tshi);
%plot(vth_pp,dvdt_vth_pp,'r*')
      
or_v = v_pp(tshi-10:tshi+10);
or_dvdt = dvdt_pp(tshi-10:tshi+10);  %%% 10 points before and after Vthresh point
%plot(or_v,or_dvdt,'ro')
p = polyfit(or_v,or_dvdt,1);   %% fitting line to data
yfit = p(1)*([vth_pp-5 vth_pp+5]) + p(2);
%plot([vth_pp-5 vth_pp+5],yfit,'r-')
or1 = p(1);
title(['OR = ' num2str(or1)])
 
%%%%% d2V/dt2 iAP

d2V = diff(vf2);
d2Vdt2 = d2V./(si./1000);
t_d2Vdt2 = t1(1:length(t1)-1);
v_d2Vdt2 = vf(1:(end-2));
figure(2)
subplot(5,1,3)
plot(t_d2Vdt2,d2Vdt2,'b-')
title(['d2V/dt2 iAP'])
xlabel('ms');ylabel('V2/s')
hold on

figure(3)
subplot(2,2,2)
plot(v_d2Vdt2,d2Vdt2)
axis square
title('2nd PP')

%%% 3rd derivative!

vf3 = filter(b,a,d2Vdt2);
figure(2)
subplot(5,1,3)
plot(t_d2Vdt2-(tstep.*(window-1)/2),vf3,'m-')

subplot(5,1,4)
d3v = diff(vf3)./(si./1000);    
t3 = t_d2Vdt2(1:(end-1));
plot(t3,d3v)
title('3rd derivative')
hold on

figure(3)
subplot(2,2,3)
plot(vf(1:(end-3)),d3v)
axis square
title('3rd PP')

%%%% and the 4th!

vf4 = filter(b,a,d3v);
figure(2)
subplot(5,1,4)
plot(t3-(tstep.*(window-1)/2),vf4,'m-')

subplot(5,1,5)
d4v = diff(vf4)./(si./1000);    
t4 = t3(1:(end-1));
plot(t4,d4v)
title('4th derivative')

figure(3)
subplot(2,2,4)
plot(vf(1:(end-4)),d4v)
axis square
title('4th PP')


%%%% output

first_spike_properties = [j istep(j) vhold(j) v_thresh vmax vmax-v_thresh whh mx_dvdt or1]'
    
%clipboard('copy',first_spike_properties);
