%%% takes in abf file ('expt') of 500ms current injection steps of increasing
%%% amplitude, determines multiple spiking 
%%% measures.  'cm' is used to normalise current inputs as pA/pF

function [] = galliano_AP_500ms(expt,cm)

close all

th = 0;
multiples = 1;
prepulse = 0.1;   
[d,si] = abf2load(expt);  %%% loads in data from .abf file into matrix d, with sample interval si (in microsec)
%%%% structure of d
%%%% 1st dimension is data points in sweep, 2nd is no of channels, 3rd is
%%%% no of sweeps
s = size(d);
nsweeps = s(3);

pAstart=0;
pAstep=10;

for n = 1:nsweeps
    
    if s(2) == 1 % so if output is only in voltage channel; requires pAstep & pAstart inputs, and can only deal in relative current changes
        
        t{n} = [0:si:(si*(s(1)-1))]'./1000000; %%% time#1 array; doesn't need to be built on sweep-by-sweep basis, but is safest.  Does need to be inverted to match voltage array from d
        v{n} = (d(:,1,n)./1000); %%% voltage array      
        t2{n} = t{n}; %%% time#2 array; stupid to have this here, but safest for historical Heka analysis reasons
        
        step_a(n) = pAstart + ((n-1)*pAstep);
        
        %plot(t{n},v{n})
        
        %pause
        
    elseif s(2) == 2
        
        %%% build up arrays in here for dual-channel recordings with both v
        %%% and amps
        
        t{n} = [0:si:(si*(s(1)-1))]'./1000000; %%% time#1 array; doesn't need to be built on sweep-by-sweep basis, but is safest.  Does need to be inverted to match voltage array from d
        v{n} = (d(:,1,n)./1000); %%% voltage array      
        t2{n} = t{n}; %%% time#2 array; stupid to have this here, but safest for historical Heka analysis reasons
        a{n} = (d(:,2,n)./1000000000000);
        
        bas_i = find(t{n}<prepulse);
        
        step_i = find((t{n}>(prepulse+0.05))&(t{n}<(prepulse+0.45)));
        hold_a(n) = mean(a{n}(bas_i))*1000000000000;
        step_a(n) = round(mean(a{n}(step_i))*1000000000000 - hold_a(n));
        
        if step_a(n)==0 & n>1     
            step_a(n) = pAstart + ((n-1)*pAstep);
        end
    end
    
    figure(7)
    plot(t{n},v{n})
    axis([0 0.8 -0.1 0.1])
    hold on
    %stop
    
    bas_i = find(t{n}<prepulse);
    hold_v(n) = mean(v{n}(bas_i)*1000);
    plot([0 prepulse],[-0.052 -0.052],'k--')
    plot([0 prepulse],[-0.058 -0.058],'k--')
    plot([0 prepulse],[hold_v(n)/1000 hold_v(n)/1000],'m-')
    spikes(n) = 0;
    disp(['reading sweep no ' num2str(n) ' Vhold = ' num2str(hold_v(n)) ' mV'])
    hold off
    %pause
    
    w = 10;  %%%% sets no of points over which to take local dvdt
    
    for j = 1:(s(1)-(w+1))
        xdiff{n}(j) = t{n}(j+w) - t{n}(j);
        ydiff{n}(j) = v{n}(j+w) - v{n}(j);
        dvdt{n}(j) = ydiff{n}(j) ./ xdiff{n}(j);
    end
    
    platv(n) = min(v{n}(find((t{n}>(prepulse+0.4))&(t{n}<(prepulse+0.5))))); %%% is plateau voltage
        
    
    if (max(v{n})>th) %& (abs(th - platv(n))>0.006)  %%%% so if there's a spike, and plateau voltage isn't too close to detection threshold
        
        spikes(n) = 1;
        
        figure(1)
        subplot(3,1,1)
        plot(t{n},v{n},'b-')
        axis([0 0.8 -0.1 0.1])
        hold on
        plot([prepulse+0.4 prepulse+0.5],[platv(n) platv(n)],'m-')
        %title(abs(th - platv(n))*1000)
        plot([0 0.8],[th th],'b:')
        subplot(3,1,3)
        %plot(t{n},a{n},'g-')
        subplot(3,1,2)
        plot(t{n}(1:length(t{n})-(w+1)),dvdt{n},'m-')
        hold on   
        base_i{n} = find(t{n}<prepulse);base(n) = mean(dvdt{n}(base_i{n}));
        %([0 0.1],[base base],'b-')
        st(n) = base(n) + 10*std(dvdt{n}(base_i{n}));
        plot([0 0.8],[st(n) st(n)],'g-')
        hold off
                
        %%% finding no of spikes by crossings of threshold line
        
        spikecrosses{n} = [];     
            for j = 1:length(v{n})-1
                if v{n}(j)<=th %%% so if point is below 0mV...
                    if v{n}(j+1)>th %%% ...but next point is above 0mV...
                        if dvdt{n}(j)>0  %%%...and dvdt is +ve (helps get rid of noise)
                            spikecrosses{n} = [spikecrosses{n}; t{n}(j)];
                        end
                    end
                end
            end
        if length(spikecrosses{n})>1
            spikecrossdiffs{n} = [];
            for j = 1:length(spikecrosses{n})-1
                spikecrossdiffs{n} = [spikecrossdiffs{n};spikecrosses{n}(j+1)-spikecrosses{n}(j)];
            end
            j = find(spikecrossdiffs{n}<0.005);  %%% so looking for spike crosses less than 5ms apart...
            %if length(j)>0
                %cdi{n} = setdiff([1:length(spikecrosses{n})],j);
                %spikecrosses{n} = spikecrosses{n}(cdi{n}); %%% so removing any 'spikes' within 5ms of each other
            %end
        end
            
        nspikes(n) = length(spikecrosses{n});
        
        %subplot(3,1,1)
        %plot(spikecrosses{n},[zeros(length(spikecrosses{n}))+th],'ro')
        
        %%%% getting measures of spike timing, and peak amplitude
        
        stwpre = 0.002; %%% sets time before crossing for analysis window for each spike
        stwpost = 0.002; %%% sets time after crossing for analysis window for each spike 
        
        for j = 1:nspikes(n)    %%%% so working through each spike in turn
            
            tws{n}(j) = [spikecrosses{n}(j)-stwpre];   %%% spike time window start
            twe{n}(j) = [spikecrosses{n}(j)+stwpost];    %%% spike time window end
            sin{n}{j} = find((t{n}>=tws{n}(j))&(t{n}<twe{n}(j)));  %index of spike window
            vspike{n}{j} = v{n}(sin{n}{j});
            tspike{n}{j} = t{n}(sin{n}{j});
            [pkspike{n}(j) pktispike{n}(j)] = max(vspike{n}{j});    %%% so finds peak voltage (and index) of each spike
            pktspike{n}(j) = tspike{n}{j}(pktispike{n}(j)); %%% and here finds time of each spike peak
            subplot(3,1,1)
            plot(pktspike{n}(j),pkspike{n}(j),'go')
            
        end
        
        bin1_spikeno(n) = length(find((pktspike{n}>=0.1)&(pktspike{n}<0.2)));
        bin2_spikeno(n) = length(find((pktspike{n}>=0.2)&(pktspike{n}<0.3)));
        bin3_spikeno(n) = length(find((pktspike{n}>=0.3)&(pktspike{n}<0.4)));
        bin4_spikeno(n) = length(find((pktspike{n}>=0.4)&(pktspike{n}<0.5)));
        bin5_spikeno(n) = length(find((pktspike{n}>=0.5)&(pktspike{n}<0.6)));
        
        hold off
        
        %pause
        
        if nspikes(n)>1
            for k = 1:(nspikes(n)-1)
                tdspike{n}(k) = pktspike{n}(k+1) - pktspike{n}(k);
            end
            tds1(n) = tdspike{n}(1);
            instf(n) = 1/tds1(n); %%% instantaneous spike frequency from 1st 2 spikes
            spkrange(n) = sum(tdspike{n});  %%%% time between 1st and last spikes in s
            spkfreq(n) = nspikes(n)./spkrange(n);
            %pause
        else
            instf(n) = 0;
            spkfreq(n) = 0;
        end
        
        if nspikes(n)>2
            %disp(tdspike{n}')
            vr(n) = std(tdspike{n}) ./ mean(tdspike{n}); %%%% measure of spike timing regularity (near 0 - regular)
            %pause
        else
            vr(n) = 999;
        end
        
        
        %%% ahp, threshold, Vmax and whh from 1st spike fired 
        
        if nspikes(n) >= 1 & sum(nspikes(1:n-1)) == 0  %% so first sweep with spike
        %if n==20  %%% or setting a specific sweep
            
            if nspikes(n)>1
                spike1_i = find((t{n}>prepulse)&(t{n}<(pktspike{n}(2)-0.005)));
            else
                spike1_i = find((t{n}>prepulse)&(t{n}<(prepulse+0.5)));   %%% narrowing window to 1st spike
            end
            spike1_v = v{n}(spike1_i);
            %disp(['length spike1_v = ' num2str(length(spike1_v))]);
            spike1_t = t{n}(spike1_i);
            figure(4)
            subplot(3,1,1)
            plot(spike1_t,spike1_v)
            hold on
            [vmax(n) vmaxi] = max(spike1_v);
            t_vmax = spike1_t(vmaxi);
            plot(t_vmax,vmax(n),'g*')
            
            %%%% dVdt  (with smoothing...)
            
            tstep = mean(diff(t{n}));
            
            spike1_v = spline(spike1_t,spike1_v,[spike1_t(1):0.000005:spike1_t(end)]);  %%%% spline fit with 5us sample intervals
            %disp(['length spline spike1_v = ' num2str(length(spike1_v))]);
            
            window = 20; %%% size of sliding window (at 20 for spline-fitted at 5us)
            af = 1; b = ones(1,window)./window;
            vf = filter(b,af,spike1_v); %%% sliding mean
            %disp(['length filtered spline spike1_v = ' num2str(length(vf))]);
            vf = vf(10:length(vf));
            %disp(length(vf))
                        
            tees = spike1_t(10:length(spike1_t));
            tf = tees-(tstep.*(window-1)/2);
            vs = spike1_v(10:length(spike1_v));
            plot(tf,vf,'m-')
              
            dv = diff(vf);
            dVdt = dv ./ tstep;   %%% 1st derivative of smoothed trace
            t1 = tf(1:length(tf)-1); 
            v1 = vs(1:length(vs)-1);
            figure(5)
            plot(v1,dVdt)   %%%%%%%%% smoothed phase plot
            figure(4)
            subplot(3,1,2)
            plot(t1,dVdt)
            hold on
            xlabel('s');ylabel('V/s')
            [mx_dVdt(n) mxi_dVdt] = max(dVdt);
            tmx_dVdt = t1(mxi_dVdt);    
            title(['max dV/dt ' num2str(mx_dVdt(n))])
            plot(tmx_dVdt,mx_dVdt(n),'*c')
            tsh_i = find((t1<tmx_dVdt)&(dVdt<10));    %%% so threshold where 1st derivative goes above 10mV/ms
            tshi = tsh_i(length(tsh_i))
            plot(t1(tshi),10,'g*')
            t_thresh = t1(tshi);    
            v_thresh(n) = v1(tshi);
            subplot(3,1,1)
            plot(t_thresh,v_thresh(n),'g*')
            hh = v_thresh(n)+ (0.5.*(vmax(n)-v_thresh(n)));
            hhv1i = find((spike1_t>t_thresh)&(spike1_v>=hh)); hh1i = hhv1i(1);
            hhv2i = find((spike1_t>spike1_t(vmaxi))&(spike1_v<=hh)); hh2i = hhv2i(1); 
            hhv1t = spike1_t(hh1i);
            hhv2t = spike1_t(hh2i);
            plot([hhv1t hhv2t],[hh hh],'g:')
            whh(n) = hhv2t-hhv1t;
            title(['sweep ' num2str(n) '  vthresh ' num2str(v_thresh(n)*1000) '  vmax ' num2str(vmax(n)*1000) '  whh ' num2str(whh(n)*1000)])
            ahpi = find(spike1_t>t_vmax);
            v_ahp = spike1_v(ahpi);
            t_ahp = spike1_t(ahpi);
            [ahp(n) ahp_i] = min(v_ahp);
            ahpt = t_ahp(ahp_i);
            plot(ahpt,ahp(n),'g*')
            rel_ahp(n) = v_thresh(n) - ahp(n);
            hold off
            
            %%%%% d2Vdt2
            
            vf2 = filter(b,af,dVdt); %%% sliding mean
            vf2 = vf2(10:length(vf2));  %%% first smoothed section often erroneous, so removed
            t_dvdt = t1(10:length(t1));
            tf2 = t_dvdt-(tstep.*(window-1)/2);  %%% offsetting filtered trace
            subplot(3,1,2)
            plot(tf2,vf2,'m-')
            hold off
            
            dv2 = diff(vf2);
            d2Vdt2 = dv2./tstep;
            t_2 = tf2(1:length(tf2)-1);
            subplot(3,1,3)
            plot(t_2,d2Vdt2)
                       
                
            spike_output = [hold_v(n) step_a(n) v_thresh(n)*1000 vmax(n)*1000 mx_dVdt(n) whh(n)*1000 ahp(n)*1000 rel_ahp(n)*1000]'

                
            hold off
            pause                
            
        end
        
        
    else
        instf(n) = 0;
        nspikes(n) = 0;
        vr(n) = 999;
    end
    
end


basei = find((hold_v<-57)&(hold_v>-63));
nonbasei = setdiff([1:n],basei);

sweeps = [1:n]; sweeps = sweeps(basei);

figure(2)
subplot(2,1,1)
plot(step_a(basei),spikes(basei),'go')
hold on
plot(step_a(nonbasei),spikes(nonbasei),'ro')
hold off
subplot(2,1,2)
plot(step_a(basei),hold_v(basei),'go')
hold on
plot(step_a(nonbasei),hold_v(nonbasei),'ro')
plot([0 step_a(length(step_a))],[-58 -58],'b--')
plot([0 step_a(length(step_a))],[-52 -52],'b--')
hold off

athresh = [];
athreshi = [];
for n = 1:(length(step_a)-1)
   if spikes(n)==0
       if spikes(n+1)==1
           athresh = [athresh;step_a(n+1)];
           athreshi = [athreshi;n+1];
       end
   end
end
athreshv = hold_v(athreshi);
athreshvdiff = abs(athreshv + 60);
[mn imin] = min(athreshvdiff);
threshi = athresh(imin);    %%%% so finding current threshold for spiking, when holding voltage is nearest to -60mV
vthreshi = athreshv(imin);
athreshi = athreshi(imin);

%disp([''])
%disp(['first spike trace = ' num2str(athreshi)])
%disp(['current threshold = ' num2str(threshi) 'pA'])
%disp(['holding voltage @ current threshold = ' num2str(vthreshi) 'mV'])

%i_input = step_a'   %%%% just displaying for copy+paste
bases = hold_v(basei)';
i_input_pA = step_a(basei)'
i_input_cm = [step_a./cm]';
i_input_cm = i_input_cm(basei)
nspikes_output = nspikes(basei)'
%instf_output = instf(basei)'
%bin1_output = bin1_spikeno(basei)'
%bin2_output = bin2_spikeno(basei)'
%bin3_output = bin3_spikeno(basei)'
%bin4_output = bin4_spikeno(basei)'
%bin5_output = bin5_spikeno(basei)'
%spkfreq_output = spkfreq(basei)'


vr = vr(basei);
vr = vr(find(vr~=999))';
vr = mean(vr);
%disp(['vr = ' num2str(vr)])

maxspikeno = max(nspikes)
cv = vr

