
function [] = ts_abf(expt,savefile)

close all

[d,si] = abf2load(expt);  %%% loads in data from .abf file into matrix d, with sample interval si (in microsec)
%%%% structure of d
%%%% 1st dimension is data points in sweep, 2nd is no of channels, 3rd is
%%%% no of sweeps
s = size(d);
nsweeps = s(3);


t = [0:si:(si*(s(1)-1))]'./1000000; %%% time#1 array
means = mean(d,3); %%% taking mean across all sweeps


if s(2) == 1    %%%% so if only current recorded
    
    i = means;
    
    vbase = 0;
    vstep = -0.01;  %%% usually -0.01 for 10mV hyperpolarising step
    
elseif s(2) == 2
    
    i = means(:,1);
    v = means(:,2);
    
    basej = find((t>0.005)&(t<0.01));
    base_v = v(basej);
    vbase = mean(base_v);
    
    platj = find ((t>0.011)&(t<0.0206));
    plat_v = v(platj);
    platv =mean(plat_v);
    
    vstep = (platv - vbase)./1000
  
end

plot(t,i,'k-')
axis square



%%%%%%%%%%%% series resistance

pk1j = find(t<1.1);
pk1_t = t(pk1j);
pk1_i = i(pk1j);
[pk1,pk1k] = min(pk1_i);
pk1t = pk1_t(pk1k);

hold on
plot(pk1t,pk1,'m*')

basej = find((t>0.005)&(t<.01));
base_t = t(basej);
base_i = i(basej);
base = mean(base_i)

plot([base_t(1) base_t(length(base_t))],[base base],'-m');


amp1 = abs(pk1 - base);

rs = abs(vstep / (amp1/1000000000000)) / 1000000

%%%%%%%%% input resistance

platj = find ((t>0.02)&(t<0.0206));
plat_t = t(platj);
plat_i = i(platj);
plat = mean(plat_i);

plot([plat_t(1) plat_t(length(plat_t))],[plat plat],'-m');

amp2 = abs(plat - base);

rm = abs(vstep / (amp2/1000000000000)) / 1000000

title(['Rs = ' num2str(rs) ' Rm = ' num2str(rm)])
axis([0 0.03 pk1-(0.1*amp1) base+(0.1*amp1)])

%%%%%%% membrane capacitance1 - area measure

areabox = abs(vstep * (amp2/1000000000000));

areaj = find((t>0.0106)&(t<0.0206));
areat = t(areaj);
areai = (i(areaj)-base)./1000000000000;
area = 0;
for n = 1:((length(areai)-1))
    area = area + (areai(n).*(areat(n+1)-areat(n))) + (0.5*((areai(n+1)-areai(n))*(areat(n+1)-areat(n))));
end

area = abs(area) - areabox;

cm = abs(area ./ vstep) * 1000000000000


%%%%%%% writing data to file

if nargin > 1 & savefile
    fid = fopen(savefile,'at');
    fprintf(fid,'%s\t %6.3f\t %6.3f\t %6.3f\t %6.3f\n', expt, base, rs, rm, cm);
    fclose(fid);
end

%close all



